// @(#)root/tmva $Id: TMVAClassification.C 40158 2011-07-07 12:36:46Z stelzer $
/**********************************************************************************
 * Project   : TMVA - a ROOT-integrated toolkit for multivariate data analysis    *
 * Package   : TMVA                                                               *
 * Root Macro: TMVAClassification                                                 *
 *                                                                                *
 * This macro provides examples for the training and testing of the               *
 * TMVA classifiers.                                                              *
 *                                                                                *
 * As input data is used a toy-MC sample consisting of four Gaussian-distributed  *
 * and linearly correlated input variables.                                       *
 *                                                                                *
 * The methods to be used can be switched on and off by means of booleans, or     *
 * via the prompt command, for example:                                           *
 *                                                                                *
 *    root -l ./TMVAClassification.C\(\"Fisher,Likelihood\"\)                     *
 *                                                                                *
 * (note that the backslashes are mandatory)                                      *
 * If no method given, a default set of classifiers is used.                      *
 *                                                                                *
 * The output file "TMVA.root" can be analysed with the use of dedicated          *
 * macros (simply say: root -l <macro.C>), which can be conveniently              *
 * invoked through a GUI that will appear at the end of the run of this macro.    *
 * Launch the GUI via the command:                                                *
 *                                                                                *
 *    root -l ./TMVAGui.C                                                         *
 *                                                                                *
 **********************************************************************************/

#include <cstdlib>
#include <iostream>
#include <map>
#include <string>

#include "TChain.h"
#include "TFile.h"
#include "TTree.h"
#include "TString.h"
#include "TObjString.h"
#include "TSystem.h"
#include "TROOT.h"

#include "TMVAGui.C"

#if not defined(__CINT__) || defined(__MAKECINT__)
// needs to be included when makecint runs (ACLIC)
#include "TMVA/Factory.h"
#include "TMVA/Tools.h"
#include "TMVA/Types.h"
#endif

void TMVAClassificationGaussMixture( )
{
   // The explicit loading of the shared libTMVA is done in TMVAlogon.C, defined in .rootrc
   // if you use your private .rootrc, or run from a different directory, please copy the
   // corresponding lines from .rootrc

   // methods to be processed can be given as an argument; use format:
   //
   // mylinux~> root -l TMVAClassification.C\(\"myMethod1,myMethod2,myMethod3\"\)
   //
   // if you like to use a method via the plugin mechanism, we recommend using
   //
   // mylinux~> root -l TMVAClassification.C\(\"P_myMethod\"\)
   // (an example is given for using the BDT as plugin (see below),
   // but of course the real application is when you write your own
   // method based)

   //---------------------------------------------------------------
   // This loads the library
   TMVA::Tools::Instance();


   // Create a ROOT output file where TMVA will store ntuples, histograms, etc.
   TString outfileName( "TMVA.root" );
   TFile* outputFile = TFile::Open( outfileName, "RECREATE" );

   TMVA::Factory *factory = new TMVA::Factory( "TMVAClassification", outputFile,
                                               "!V:!Silent:Color:DrawProgressBar:Transformations=I;D;P;U:AnalysisType=Classification" );

   // (TMVA::gConfig().GetVariablePlotting()).fNbinsMVAoutput = 5;
   // (TMVA::gConfig().GetVariablePlotting()).fNbins1D = 5;
   // (TMVA::gConfig().GetVariablePlotting()).fNbins2D = 5;

   // Define the input variables that shall be used for the MVA training
   // note that you may also use variable expressions, such as: "3*var1/var2*abs(var3)"
   // [all types of expressions that can also be parsed by TTree::Draw( "expression" )]
   factory->AddVariable( "var0",                "Variable 0", "units", 'F' );
   factory->AddVariable( "var1",                "Variable 1", "units", 'F' );

   // Read training and test data
   // (it is also possible to use ASCII format as input -> see TMVA Users Guide)

   TString fnameTrain = "./data/GaussMixtureSeed103_Ev10000.root";
   TString fnameTest  = "./data/GaussMixtureSeed123_Ev100000.root";

   TFile *inputTrain = TFile::Open( fnameTrain );
   TFile *inputTest  = TFile::Open( fnameTest );
   
   std::cout << "--- TMVAClassification       : Using input Train file: " << inputTrain->GetName() << std::endl;
   std::cout << "--- TMVAClassification       : Using input Test  file: " << inputTest->GetName() << std::endl;
   
   // --- Register the training and test trees

   TTree *signalTrain     = (TTree*)inputTrain->Get("TreeS");
   TTree *backgroundTrain = (TTree*)inputTrain->Get("TreeB");
   TTree *signalTest      = (TTree*)inputTest->Get("TreeS");
   TTree *backgroundTest  = (TTree*)inputTest->Get("TreeB");
   
   // global event weights per tree (see below for setting event-wise weights)
   Double_t signalWeight     = 1.0;
   Double_t backgroundWeight = 1.0;
   
   // You can add an arbitrary number of signal or background trees
   factory->AddSignalTree    ( signalTrain,     signalWeight     , TMVA::Types::kTraining);
   factory->AddBackgroundTree( backgroundTrain, backgroundWeight , TMVA::Types::kTraining);
   factory->AddSignalTree    ( signalTest,     signalWeight     , TMVA::Types::kTesting);
   factory->AddBackgroundTree( backgroundTest, backgroundWeight , TMVA::Types::kTesting);
   

   //   factory->SetSignalWeightExpression( "weight" );
   //   factory->SetBackgroundWeightExpression( "weight" );

   // Apply additional cuts on the signal and background samples (can be different)
   TCut mycuts = ""; // for example: TCut mycuts = "abs(var1)<0.5 && abs(var2-0.5)<1";
   TCut mycutb = ""; // for example: TCut mycutb = "abs(var1)<0.5";

   factory->PrepareTrainingAndTestTree( mycuts, mycutb,"NormMode=None:!V" );

   // ---- Book MVA methods
   //
   factory->BookMethod( TMVA::Types::kMLP, "MLP", "H:!V:NeuronType=tanh:VarTransform=N:NCycles=100:HiddenLayers=N+5:TestRate=5:!UseRegulator" );


   // factory->BookMethod( TMVA::Types::kBDT, "BDT", "!H:!V:NTrees=500" );
   
  
   
   // factory->BookMethod( TMVA::Types::kSVM, "SVM", "Gamma=0.3:Tol=0.01:VarTransform=Norm" );
   
   // factory->BookMethod( TMVA::Types::kLikelihood, "Likelihood",
   //                      "H:!V:!TransformOutput:PDFInterpol=Spline2:NSmoothSig[0]=20:NSmoothBkg[0]=20:NSmoothBkg[1]=10:NSmooth=1:NAvEvtPerBin=500" );
   
   
   // factory->BookMethod( TMVA::Types::kLikelihood, "LikelihoodD",
   // "!H:!V:TransformOutput:PDFInterpol=Spline2:NSmoothSig[0]=20:NSmoothBkg[0]=20:NSmooth=5:NAvEvtPerBin=50:VarTransform=Decorrelate" );
   


   // ---- Now you can tell the factory to train, test, and evaluate the MVAs

   // Train MVAs using the set of training events
   factory->TrainAllMethods();

   // ---- Evaluate all MVAs using the set of test events
   factory->TestAllMethods();

   // ----- Evaluate and compare performance of all configured MVAs
   factory->EvaluateAllMethods();

   // --------------------------------------------------------------

   // Save the output
   outputFile->Close();

   std::cout << "==> Wrote root file: " << outputFile->GetName() << std::endl;
   std::cout << "==> TMVAClassification is done!" << std::endl;

   delete factory;

   // Launch the GUI for the root macros
   if (!gROOT->IsBatch()) TMVAGui( outfileName );
}
